package gov.va.med.mhv.admin.service.mhvusers;

import gov.va.med.mhv.common.api.dto.PatientDTO;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.util.ResponseUtil;


/**
 * Provides Async business services for MVI Integration.  
 * Business services include operations such as correlation.
 *
 */
public interface MviAsyncIntegrationService {	
	
	/**
	 * System will try to match them with MVI and move them to PREREQUISITE Complete if everything goes fine.
	 *
	 * @param userProfile  
	 * @return VoidServiceResponse
	 */
	public void existingUserWithPendingDataMismatch(UserProfileDTO userProfile, ResponseUtil response);

	/**
	 * Execute the MviAuthenticate operation
	 *
	 * @param patient  
	 * @param userProfile  
	 * @param authenticate  
	 * @param batch  
	 * @return void
	 */
	public void mviAuthenticate(PatientDTO patient, UserProfileDTO userProfile, Boolean authenticate, Boolean batch, String versionInfo);

	/**
	 * Execute the AddPersonForCorrelation operation
	 *
	 * @param patient  
	 * @param userProfile  
	 * @param isRegistration  
	 * @return void
	 */
	public void addPersonForCorrelation(PatientDTO patient, UserProfileDTO userProfile, Boolean isRegistration, ResponseUtil response, String versionInfo);

	/**
	 * Execute the MviUnauthenticateUncorrelateForBatch operation
	 *
	 * @param patient  
	 * @param userProfile  
	 * @param adminUserName  
	 * @return void
	 */
	public void mviUnauthenticateUncorrelateForBatch(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, String versionInfo);
}
