package gov.va.med.mhv.admin.service.mhvusers;

import gov.va.med.mhv.common.api.dto.PatientDTO;
import gov.va.med.mhv.common.api.dto.UserProfileDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;


/**
 * Provides business services for MVI Integration . Business services include
 * operations such as patient lookup, registry information changes and patient
 * (VistA) correlation. Generated Code - DO NOT MODIFY
 */
public interface MviIntegrationService {

	public ResponseUtil<PatientDTO> removeCorrelationInMVI(String icn, Long userProfileId, String adminUserName) throws MHVException;
	
	public ResponseUtil<PatientDTO> deletePersonFromCorrelationForHelpDesk(String icn, Long userProfileId, String adminUserName) throws MHVException;
	
	public ResponseUtil<PatientDTO> mviAuthenticateWithVersion(PatientDTO patientId,
			UserProfileDTO userProfile, Boolean authenticate, Boolean batch, String adminUserName) throws MHVException;
	
	public void forceUncorrelation(Long patientId, Long userProfileId, String adminUserName, ResponseUtil response) throws MHVException;
	
	public ResponseUtil<PatientDTO> forceUncorrelationforUnauthUser(Long patientId, Long userProfileId, String adminUserName) throws MHVException;
	
	public ResponseUtil<PatientDTO> updateUserProfileWithVersion(String adminUserName,UserProfileDTO userProfile) throws MHVException ;
	
	public ResponseUtil<PatientDTO> matchPersonInIPAWithVersion(UserProfileDTO userProfileDTO, String adminUserName) throws MHVException ;

	public ResponseUtil<PatientDTO> addPersonForCorrelationForIPAWithVersion(PatientDTO patientDTO, UserProfileDTO userProfileDTO, String adminUserName) throws MHVException;
	
	
/*
	*//** Name which uniquely identifies the MviIntegration service. *//*
	//public static final String SERVICE_NAME = "gov.va.med.mhv.usermgmt.MviIntegrationService";

	/**
	 * Execute the GetEdipiFromMVI operation
	 *
	 * @param patient
	 * @return
	 
	public String getEdipiFromMVI(PatientDTO patient);

	*//**
	 * Execute the DeletePersonFromCorrelationForIPA operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param adminUserName
	 * @param response
	 *//*
	public void deletePersonFromCorrelationForIPA(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, ResponseUtil response);

	*//**
	 * Execute the SearchPersonInMVIForUSAA operation
	 *
	 * @param firstName
	 * @param middleName
	 * @param lastName
	 * @param birthDate
	 * @param gender
	 * @param ssn
	 * @return PatientServiceResponse
	 *//*
	public void searchPersonInMVIForUSAA(String fName, String mName, String lName, String DOB, String gender, String SSN, ResponseUtil response);
	
	*//**
	 * Execute the RetrievePersonInMVIForDSLogon operation
	 *
	 * @param edipiValue
	 * @return PatientServiceResponse
	 *//*
	public void retrievePersonInMVIForDSLogon(String edipiValue, ResponseUtil response);

	*//**
	 * Execute the AddPersonForCorrelationForIPA operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param adminUserName
	 * @return PatientServiceResponse
	 *//*
	public void addPersonForCorrelationForIPA(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, ResponseUtil response, String versionInfo);

	*//**
	 * Execute the UpdateTreatmentFacilitiesForExistingPatients operation
	 *
	 * @param patient
	 * @return PatientServiceResponse
	 *//*
	public void updateTreatmentFacilitiesForExistingPatients(PatientDTO patient, ResponseUtil response);



	*//**
	 * Execute the MviAuthenticate operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param authenticate
	 * @param batch
	 * @return BooleanServiceResponse
	 *//*
	public void mviAuthenticate(PatientDTO patient, UserProfileDTO userProfile, Boolean authenticate, Boolean batch, ResponseUtil response, String versionInfo);

	*//**
	 * Execute the DeletePersonFromCorrelationForHelpDesk operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param adminUserName
	 * @return PatientServiceResponse
	 *//*
	public void deletePersonFromCorrelationForHelpDesk(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, ResponseUtil response);

	*//**
	 * Execute the SearchPersonInMVI operation
	 *
	 * @param userProfile
	 * @param atRegistration
	 * @return PatientServiceResponse
	 *//*
	public void searchPersonInMVI(UserProfileDTO userProfile, Boolean atRegistration, ResponseUtil respons, String versionInfo);

	*//**
	 * Execute the UpdateUserProfile operation
	 *
	 * @param userProfile
	 * @return PatientServiceResponse
	 
		public PatientServiceResponse updateUserProfile(UserProfile userProfile);

		

	/**
	 * Execute the SearchPersonInMVIByICN operation
	 *
	 * @param icn
	 * @return PatientServiceResponse
	 *//*
	public void searchPersonInMVIByICN(String icn, ResponseUtil response);


	*//**
	 * Execute the SearchPersonInMVIByICN operation
	 *
	 * @param icn
	 * @reurn 
	 *//*
	public HashMap<String, Object> search1305PersonInMVIByICN(String icn, ResponseUtil response);
	
	*//**
	 * Execut the SearchPersonInMVIByICNAccountSync operation
	 *
	 *  @param icn
	 *  @param response
	 *//*
	public void searchPersonInMVIByICNAccountSync(String icn, ResponseUtil response);

	*//**
	 * Execute the MviUnauthenticateUncorrelateForBatch operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param adminUserName
	 * @return BooleanServiceResponse
	 *//*
	public void mviUnauthenticateUncorrelateForBatch(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, ResponseUtil response);

	*//**
	 * Execute the MatchPersonInIPA operation
	 *
	 * @param userProfile
	 * @return PatientServiceResponse
	 *//*
	public void matchPersonInIPA(UserProfileDTO userProfile, ResponseUtil response, String mviProperties, String versionInfo);

	*//**
	 * Execute the ForceUncorrelation operation
	 *
	 * @param patient
	 * @param userProfile
	 * @param adminUserName
	 * @return PatientServiceResponse
	 *//*
	public void forceUncorrelation(PatientDTO patient, UserProfileDTO userProfile, String adminUserName, ResponseUtil response);

	*//**
	 * Execute the SearchPersonInMVIForDSLogon operation
	 *
	 * @param edipiValue
	 * @return PatientServiceResponse
	 *//*
	public void searchPersonInMVIForDSLogon(String edipiValue, ResponseUtil response);

	*//**
	 *
	 * @param userProfile
	 * @param fccxIcn
	 * @param response
	 *//*
	public void createPatientForFccx(UserProfileDTO userProfile, String fccxIcn, ResponseUtil response);
	
	
	public void addPersonForCorrelationAPI(PatientDTO patient, UserProfileDTO userProfile, ResponseUtil response, String versionInfo);*/
}
