package gov.va.med.mhv.admin.service.impl.calendar;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.Path;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import gov.va.med.mhv.admin.converter.CalendarCategoryConverter;
import gov.va.med.mhv.admin.dto.CalendarCategoryDTO;
import gov.va.med.mhv.admin.dto.CalendarEventDTO;
import gov.va.med.mhv.admin.model.CalendarCategory;
import gov.va.med.mhv.admin.repository.CalendarCategoryRepository;
import gov.va.med.mhv.admin.service.calendar.CalendarCategoryService;
import gov.va.med.mhv.admin.validator.CalendarCategoryValidator;
import gov.va.med.mhv.admin.validator.CalendarEventValidator;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;

@Path("/")
@Service("calendarCategoryService")
@Component
public class CalendarCategoryServiceImpl implements CalendarCategoryService {
	private static Logger log = LogManager.getLogger(CalendarCategoryServiceImpl.class);

	@Autowired
	private CalendarCategoryRepository calendarCategoryRepository;
	
	@Autowired
	private CalendarCategoryValidator validator;

	@Override
	public List<CalendarCategoryDTO> getAllActiveCategories() throws MHVException {
		List<CalendarCategoryDTO> calendarCategoryDtos = null;
		List<CalendarCategory> calendarCategories = null;

		try {
			calendarCategories = calendarCategoryRepository.getAllActiveCategories();
		} catch (Exception e) {
			log.error("Error in fetching all active categories " + e);
			throw new MHVException(e);
		}

		if (calendarCategories == null || calendarCategories.size() == 0) {
			calendarCategories = new ArrayList<CalendarCategory>();
		} else {
			calendarCategoryDtos = new CalendarCategoryConverter().convert(calendarCategories);
		}

		return calendarCategoryDtos;
	}


	@Override
	public void deleteCalendarCategory(Long categoryId) throws MHVException {
		
		try{
			if(categoryId == null){
				throw new MHVException("Invalid Category Id");
			}
			CalendarCategory category=calendarCategoryRepository.findOne(categoryId);
			category.setDeleted(true);
			calendarCategoryRepository.save(category);
		}catch(Exception e){
			throw new MHVException(e);
		}
	}


	@Override
	public void saveCalendarCategory(CalendarCategoryDTO categoryDTO) throws MHVException {
		try{
			if(categoryDTO == null || categoryDTO.getCategoryName() == null){
				throw new MHVException("Invalid Category Data");
			}	
			ResponseUtil<CalendarCategoryDTO> response = new ResponseUtil<CalendarCategoryDTO>();
			response = validator.fieldValidations(categoryDTO);
			if (response.getValidationErrors().size() > 0) {
				response.setFailure(true);
			}

			if (response.isFailure()) {
				log.error(response.getFailureMessage());
				throw new MHVException(response);
			}
			
			CalendarCategoryConverter converter = new CalendarCategoryConverter();
			CalendarCategory calendarCategory = converter.convert(categoryDTO);
			calendarCategoryRepository.save(calendarCategory);
		}catch(MHVException e){
			throw e;
		}catch(Exception e){
			throw new MHVException(e);
		}
		
	}
}
