package gov.va.med.mhv.admin.service.impl.mhvusers;

import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import gov.va.med.mhv.admin.converter.FacilityInfoConverter;
import gov.va.med.mhv.admin.exception.MHVRuntimeException;
import gov.va.med.mhv.admin.repository.FacilityInfoRepository;
import gov.va.med.mhv.admin.service.mhvusers.FacilityInfoService;
import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.data.model.FacilityInfo;

@Path("/")
@Service("facilityInfoService")
@Component
public class FacilityInfoServiceImpl implements FacilityInfoService {

	protected static Logger log = LogManager.getLogger(FacilityInfoServiceImpl.class);

	@Autowired
	private FacilityInfoRepository facilityInfoRepository;
	
	@Autowired
	private FacilityInfoConverter facilityInfoConverter;
	
	@Override
	@GET
	@Path("/getFacilityInfoById/{facilityInfoId}")
	@Produces(MediaType.APPLICATION_JSON)
	public FacilityInfoDTO getFacilityInfoById(@PathParam("facilityInfoId") Long facilityInfoId) throws MHVException {
		FacilityInfoDTO facilityInfoDTO = new FacilityInfoDTO();
		try {
			FacilityInfo facilityInfo = facilityInfoRepository.getFacilityInfoById(facilityInfoId);

			if (facilityInfo != null) {
				facilityInfoDTO = facilityInfoConverter.convert(facilityInfo);
			} else {
				throw new MHVException("UserProfile Not Found for the userProfileId::" + facilityInfoId);
			}
		} catch (Exception e) {
			throw new MHVException(e);
		}
		return facilityInfoDTO;
	}
	

	@Override
	public FacilityInfoDTO saveFacilityInfo(FacilityInfoDTO facilityInfo) {
		FacilityInfo storedFacilityInfo = null;
		try {
			storedFacilityInfo = facilityInfoConverter.convertToDomain(facilityInfo);
			facilityInfoRepository.save(storedFacilityInfo);
		} catch (Exception e) {
			log.error("Error in storing FacilityInfo details : ", e);
			throw new MHVRuntimeException("Error in storing FacilityInfo details");
		}
		return facilityInfoConverter.convert(storedFacilityInfo);
	}

	@Override
	public List<FacilityInfoDTO> getAllFacilityInfos() {
		List<FacilityInfo> facilityInfos = null;
		try {
			facilityInfos = facilityInfoRepository.getAllFacilityInfos();
		} catch (Exception e) {
			log.error("Error in fetching getAllFacilityInfos() ", e);
			throw new MHVRuntimeException("Error in fetching getAllFacilityInfos()");
		}
		return facilityInfoConverter.convertList(facilityInfos);
	}

	@Override
	public List<FacilityInfoDTO> queryByStationNumber(String stationNumber) {
		List<FacilityInfo> facilityInfos = null;
		try {
			facilityInfos = facilityInfoRepository.queryByStationNumber(stationNumber);
		} catch (Exception e) {
			log.error("Error in fetching queryByStationNumber() " + stationNumber, e);
			throw new MHVRuntimeException("Error in fetching queryByStationNumber()");
		}
		return facilityInfoConverter.convertList(facilityInfos);
	}

}
