package gov.va.med.mhv.admin.web.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.TimeZone;

import javax.annotation.PostConstruct;
import javax.el.ValueExpression;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.component.schedule.Schedule;
import org.primefaces.event.SelectEvent;
import org.primefaces.event.data.SortEvent;
import org.primefaces.model.DefaultScheduleModel;
import org.primefaces.model.LazyScheduleModel;
import org.primefaces.model.ScheduleModel;

import gov.va.med.mhv.admin.dto.CalendarCategoryDTO;
import gov.va.med.mhv.admin.dto.CalendarEventDTO;
import gov.va.med.mhv.admin.dto.CalendarSearchCriteriaDTO;
import gov.va.med.mhv.admin.dto.CategoryDTO;
import gov.va.med.mhv.admin.dto.MHVEventModel;
import gov.va.med.mhv.admin.dto.UserCalViewDTO;
import gov.va.med.mhv.admin.dto.VisnDTO;
import gov.va.med.mhv.admin.util.CommonUtility;
import gov.va.med.mhv.admin.util.MHVEventComparator;
import gov.va.med.mhv.common.api.exception.MHVException;

@ManagedBean(name = "healthCalendarController")
@ViewScoped
public class HealthCalendarController extends AbstractController {

	private static final long serialVersionUID = 2502698156997262683L;
	private MHVEventModel event = new MHVEventModel();
	private MHVEventModel selectedEvent = new MHVEventModel();
	private MHVEventModel newEvent = new MHVEventModel();
	private ScheduleModel eventModel;
	private ScheduleModel lazyEventModel;
	private Schedule schedule = new Schedule();
	private String scheduleView;
	private Date scheduleDate;
	protected int eventRowsPerPage = 10;
	protected int searchResultsRowsPerPage = 10;
	protected ValueExpression eventSortColumn;
	protected String eventsortBy;
	private List<CategoryDTO> calCategories;
	private String[] selectedCategories = new String[50];
	private CalendarSearchCriteriaDTO searchCriteria;
	List<MHVEventModel> events;
	private DataTable eventTable;
	private UserCalViewDTO userPreference;
	private String viewType;
	private String previousView;
	private String userPreferView = "";
	private List<MHVEventModel> searchResults;
	private Date selectDate;

	private Date minCurrentYear;
	private Date maxCurrentYear;
	private Date minNextYear;
	private Date maxNextYear;
	private String minCalDate;
	private String maxCalDate;
	private Integer searchResultNumber;
	private String lastupdatedDate;

	private Date exportStartDate;
	private Date exportEndDate;
	private String[] exportSelectedCategories = new String[50];
	private List<CategoryDTO> exportCalCateories;
	private Date exportMinYear;
	private Date exportMaxYear;
	private boolean exportValidationError;
	private String view;
	private String lastViewType = "";
	Long userprofileId;
	private List<VisnDTO> visns;
	
	
	private CalendarCategoryDTO newCategory = new CalendarCategoryDTO();
	private CalendarCategoryDTO selectedCategory= new CalendarCategoryDTO();
	private Date viewDisplayDate;
	
	private static final String AGENDA_WEEK="agendaWeek";
	private static final String AGENDA_DAY="agendaDay";
	private static final String MONTH="month";
	
	private String listselection="curr_month";

	private static Logger log = LogManager.getLogger(HealthCalendarController.class);

	public String resetCalendar() {
		
		eventModel = new LazyScheduleModel(){
			
			@Override 
			public void loadEvents(Date start, Date end) { 
				resetMessages();
				if (events != null) {
					for (MHVEventModel event : events) {
						eventModel.addEvent(event);
					}
				} 
				viewDisplayDate=getCalMonthDisplayDate(end);
				
				schedule = (Schedule) FacesContext.getCurrentInstance().getViewRoot()
						.findComponent("calendarSummaryForm:schedule");
				
				if(schedule.getView().equalsIgnoreCase(MONTH)){
					Date monthDisplayDate =getCalMonthDisplayDate(end);
					
					if(isCurrentMonth(monthDisplayDate)){
						viewDisplayDate=new Date();
					}else{
						viewDisplayDate=monthDisplayDate;
					}
				}else if(schedule.getView().equalsIgnoreCase(AGENDA_WEEK)){
					if(isCurrentWeek(start,end)){
						viewDisplayDate=new Date();
					}else{
						viewDisplayDate=getCalWeekDisplayDate(start);
					}
				}else if(schedule.getView().equalsIgnoreCase(AGENDA_DAY)){
					viewDisplayDate=getCalDayDisplayDate(start);
				}
				setScheduleDate(viewDisplayDate);
		    }
		};
		return null;
	}
	

	@PostConstruct
	public void init() {
		try {
			String adminUserName = getAdminUserName();
			schedule = (Schedule) FacesContext.getCurrentInstance().getViewRoot()
					.findComponent("calendarSummaryForm:schedule");
			
			resetMessages();
			if (adminUserName != null) {
				log.info("LOG: adminUserName is not null: " + adminUserName);
				setScheduleDate(new Date());
				exportCalendarDates();
				eventModel = new DefaultScheduleModel();
				getCalendarCategories();
				getAllVisns();
				loadSchedule(getDefaultSearchCriteria());
				loadCalendarDates();
				viewType = "calendarView";
				setScheduleView(MONTH);
			}else{
				log.info("LOG: adminUserName is NULL: " + adminUserName);				
			}
		} catch (Exception e) {
			log.error("Error initializing Calendar:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
		lastViewType = viewType;
	}

	private CalendarSearchCriteriaDTO getDefaultSearchCriteria() {
		searchCriteria = new CalendarSearchCriteriaDTO();
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setKeyword(null);
		searchCriteria.setViewType(null);
		return searchCriteria;
	}

	private CalendarSearchCriteriaDTO getCurrentMonthSearchCriteria() {
		searchCriteria = new CalendarSearchCriteriaDTO();
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setKeyword(null);
		searchCriteria.setViewType(null);
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		Date startDate = cal.getTime();
		cal.add(Calendar.MONTH, 1);
		cal.add(Calendar.DAY_OF_MONTH, -1);
		Date endDate = cal.getTime();
		searchCriteria.setRangeStart(startDate);
		searchCriteria.setRangeEnd(endDate);
		return searchCriteria;
	}

	private void loadSchedule(CalendarSearchCriteriaDTO searchCriteria) throws MHVException {
		try {
			events = calendarServiceProxy.findEventsForUser(searchCriteria);
		} catch (MHVException e) {
			log.error("Error in finding User Events:", e);
			throw e;
		}
	}

	private void getCalendarCategories() throws MHVException {
		try {
			calCategories = new ArrayList<CategoryDTO>();
			List<CategoryDTO> calCatgs = calendarServiceProxy.getAllCategories();
			int i = 0;
			if (calCatgs != null) {
				for (CategoryDTO catg : calCatgs) {
					calCategories.add(catg);
					selectedCategories[i] = catg.getCategory();
					i++;
				}
			}

		} catch (MHVException e) {
			log.error("Error in getting Calendar Categories:");
			throw e;
		}
	}
	
	private List<VisnDTO> getAllVisns() throws MHVException {
		try{
			visns=calendarServiceProxy.getAllVisns() ;
		}catch(Exception e){
			throw e;
		}
		return visns;
	}


	// *********************************Event actions start
	// ********************************************
	// TODO visit this
	public void onViewChange(SelectEvent selectEvent) {
		schedule = (Schedule) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("calendarSummaryForm:schedule");
		scheduleView = selectEvent.getObject().toString();
		resetMessages();
		viewType = "calendarView";
	}

	public void onDateSelect(SelectEvent selectEvent) {
		resetMessages();
		setScheduleView(AGENDA_DAY);
		setScheduleDate((Date) selectEvent.getObject());
		viewType = "calendarView";
	}
	

	public void addEventDisplay() {
		resetMessages();
		newEvent = new MHVEventModel();
		newEvent.setBeginDate(new Date());
		newEvent.setFinishDate(new Date());
		viewType = "addEventView";
	}

	public void onEventSelect(SelectEvent selectEvent) {
		resetMessages();
		lastViewType = viewType;
		selectedEvent = (MHVEventModel) selectEvent.getObject();
		selectedEvent = findSelectedEvent(selectedEvent.getEventId());
		setTimes(selectedEvent);
		viewType = "eventDetailView";
	}

	private MHVEventModel findSelectedEvent(Long eventId) {
		MHVEventModel event = null;
		try {
			event = calendarServiceProxy.getPersonalEventById(eventId);
		} catch (MHVException e) {
			log.error("Error in Find Event:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
		return event;
	}

	public void showEventDetail(MHVEventModel event) {
		resetMessages();
		lastViewType = viewType;
		selectedEvent = event;
		setTimes(selectedEvent);
		viewType = "eventDetailView";
	}

	private void setTimes(MHVEventModel event) {
		String dateTime = CommonUtility.dateToString(event.getBeginDate(), HOUR_MINUTE);
		if (!dateTime.equals("00:00")) {
			event.setStartHour(CommonUtility.dateToString(event.getBeginDate(), HOURS));
			event.setStartMinute(CommonUtility.dateToString(event.getBeginDate(), MINUTES));
		}
		dateTime = CommonUtility.dateToString(event.getFinishDate(), HOUR_MINUTE);
		if (!dateTime.equals("00:00")) {
			event.setEndHour(CommonUtility.dateToString(event.getFinishDate(), HOURS));
			event.setEndMinute(CommonUtility.dateToString(event.getFinishDate(), MINUTES));
		}
	}

	public void eventSummary() {
		resetMessages();
		lastViewType = viewType;
		eventTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("calendarListForm:eventList");
		currentMonth();
		viewType = "eventView";
		if (eventSortColumn != null && eventsortBy != null) {
			eventTable.setValueExpression("sortBy", eventSortColumn);
			eventTable.setSortOrder(eventsortBy);
		}
	}

	public void onEventSort(SortEvent event) {
		eventSortColumn = event.getSortColumn().getValueExpression("sortBy");
		eventsortBy = event.isAscending() ? "ascending" : "descending";
	}

	public void deleteEventDisplay() {
		resetMessages();
		viewType = "eventDeleteView";
	}

	public void cancelEventDetail() {
		viewType = "eventDetailView";
	}

	// PUBLISH the event
	public void saveEvent(Long eventId) {
		CalendarEventDTO calEvent = new CalendarEventDTO();
		MHVEventModel event = null;
		lastViewType = "calendarView";
		if (eventId != null && eventId != 0) {
			event = getSelectedEvent();
			//viewType = "eventDetailView";
		} else {
			event = getNewEvent();
			//viewType = "addEventView";
		}
		setEventTimes(event);
		convertToEventDTO(event, calEvent);
		try {
			calendarServiceProxy.saveCalendarEvent(calEvent);
			saveEventMessage = true;
			loadSchedule(getDefaultSearchCriteria());
			viewType = "calendarView";
		} catch (MHVException e) {
			log.error("Error in Save Event", e);
			super.processErrorMessages(e);
		} catch (Exception e) {
			log.error("Error in Save Event:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	private void setEventTimes(MHVEventModel event) {

		if (event.isAllDay()) {
			event.setStartHour(null);
			event.setStartMinute(null);
			event.setEndHour(null);
			event.setEndMinute(null);
		}

		Date beginDateTime = null;
		if (event.getBeginDate() != null) {
			beginDateTime = CommonUtility.getDaTeTime(event.getBeginDate(), event.getStartHour(),
					event.getStartMinute());
		}
		event.setBeginDate(beginDateTime);
		Date endDateTime = null;
		if (event.getFinishDate() != null) {
			endDateTime = CommonUtility.getDaTeTime(event.getFinishDate(), event.getEndHour(), event.getEndMinute());
		}
		event.setFinishDate(endDateTime);
	}

	private void convertToEventDTO(MHVEventModel event, CalendarEventDTO calEvent) {
		calEvent.setAllDay(event.isAllDay());
		calEvent.setDescription(event.getDescription());
		calEvent.setEndDate(event.getFinishDate());
		calEvent.setEventId(event.getEventId());
		calEvent.setLocation(event.getLocation());
		calEvent.setRecurrenceInterval(event.getRepeatInterval());
		
		calEvent.setRecurUntilDate(event.getRepeatEndDate());
		
		calEvent.setRepeating(event.getRepeatEndDate() != null);
		calEvent.setStartDate(event.getBeginDate());
		calEvent.setSubject(event.getEventName());
		calEvent.setUserProfileId(userprofileId);
		
		calEvent.setTimeZone(event.getTimeZone());
		calEvent.setLink(event.getLink());
		VisnDTO visn = new VisnDTO();
		visn.setVisnId(event.getVisnId());
		calEvent.setVisn(visn);
		CalendarCategoryDTO catDTO = new CalendarCategoryDTO();
		catDTO.setCalendarCategoryId(event.getCategoryId());
		calEvent.setCalendarCategory(catDTO);
		if(event.getVisnId().longValue()== 0){
			calEvent.setVhaEvent(true);
		}
	}

	public void deleteEvent(Long eventId) {
		lastViewType = viewType;
		try {
			calendarServiceProxy.deleteCalendarEvent(eventId);
			loadSchedule(getDefaultSearchCriteria());
			deleteEventMessage = true;
			viewType = "calendarView";
		} catch (MHVException e) {
			log.error("Error in Deleting Event", e);
			super.processErrorMessages(e);
		} catch (Exception e) {
			log.error("Error in Deleting Event:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void backToEventList() {
		resetMessages();
		// lastViewType=viewType;
		viewType = "eventView";
	}

	public void cancelEvent() {
		viewType = lastViewType;
	}
	
	public void saveCategory(Long categoryId){
		resetMessages();
		CalendarCategoryDTO category=new CalendarCategoryDTO();
		if(categoryId  != null && categoryId != 0){
			 category=getSelectedCategory();
		}else{
			category=getNewCategory();
		}
		try{
			calendarServiceProxy.saveCalendarCategory(category);
			getCalendarCategories();
			newCategory=new CalendarCategoryDTO();
			saveEventCategory=true;
			viewType = "addCategoryView";
		}catch (MHVException e) {
			log.error("Error in Save Event", e);
			super.processErrorMessages(e);
		} catch (Exception e) {
			log.error("Error in Save Event:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}
	

	public void addCategoryDisplay() {
		resetMessages();
		newCategory=new CalendarCategoryDTO();
		viewType = "addCategoryView";
	}
	
	public void editCategoryDisplay(CategoryDTO category) {
		resetMessages();
		selectedCategory.setCalendarCategoryId(category.getCategoryId());
		selectedCategory.setCategoryName(category.getCategory());
		viewType = "editCategoryView";
	}
	
	
	public void deleteCategory(Long categoryId){
		try{
			calendarServiceProxy.deleteCalendarCategory(categoryId);
			getCalendarCategories();
			deleteEventCategory=true;
			viewType = "addCategoryView";
		}catch (MHVException e) {
			log.error("Error in Delet Event Category", e);
			super.processErrorMessages(e);
		} catch (Exception e) {
			log.error("Error in Delet Event Category:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}
	

	public void backToEventDetail() {
		viewType = "eventDetailView";
	}

	public void backToGenericEventDetail() {
		viewType = "genericEventDetailView";
	}

	public void backToTaskDetail() {
		viewType = "taskDetailView";
	}

	public void backToHealth() {
		resetMessages();
		viewType = "calendarView";
		lastViewType=viewType;
	}

	// Search Events
	public void searchEvents() {
		lastViewType = viewType;
		try {
			searchResults = new ArrayList<MHVEventModel>();
			ArrayList<MHVEventModel> tmpSearchResults = new ArrayList<MHVEventModel>();
			maxSearchResults = false;
			if (searchEventText != null && searchEventText.trim().length() > 0) {
				CalendarSearchCriteriaDTO criteria = getDefaultSearchCriteria();
				criteria.setViewType("listView");
				criteria.setKeyword(searchEventText);
				List<MHVEventModel> srchResults = calendarServiceProxy.searchEventsForUser(criteria);
				if (srchResults != null && srchResults.size() > 100) {
					searchResultNumber = srchResults.size();
					searchResults = srchResults.subList(0, 100);
					maxSearchResults = true;
				} else {
					searchResults = srchResults;
				}

		//		Collections.reverse(searchResults);
				HashMap uniqueEvents = new HashMap();

				for (MHVEventModel search : searchResults) {
					String Key = search.getEventName();
					if (uniqueEvents.get(Key) == null) {
						uniqueEvents.put(Key, search);
					}

				}

				tmpSearchResults.addAll(uniqueEvents.values());

				searchResults = tmpSearchResults;

				if (searchResults != null && searchResults.size() > 100) {
					searchResultNumber = searchResults.size();
					searchResults = searchResults.subList(0, 100);
					maxSearchResults = true;
				} else {
					maxSearchResults = false;
				}

				Collections.sort(searchResults, new MHVEventComparator());
			}
			viewType = "searchResultsView";
		} catch (MHVException e) {
			log.error("Error in Searching Event", e);
			super.processErrorMessages(e);
		} catch (Exception e) {
			log.error("Error in Searching Event:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void showSearchResultDetail(MHVEventModel event) {
		resetMessages();
		lastViewType = viewType;
		selectedEvent = event;
		setTimes(selectedEvent);
		viewType = "eventDetailView";
	}

	// *********************************Event actions End
	// ********************************************

	// ******************************* AJAX Actions start
	// ***********************************************

	public void dayViewAjax() {
		lastViewType = "calendarView";
		try {
			resetMessages();
			loadSchedule(getDefaultSearchCriteria());
			setScheduleView(AGENDA_DAY);
			viewType = "calendarView";
		} catch (Exception e) {
			log.error("Error in finding Events:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void weekViewAjax() {
		lastViewType = "calendarView";
		try {
			resetMessages();
			loadSchedule(getDefaultSearchCriteria());
			setScheduleView(AGENDA_WEEK);
			viewType = "calendarView";
		} catch (Exception e) {
			log.error("Error in finding Events:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void monthViewAjax() {
		lastViewType = "calendarView";
		try {
			resetMessages();
			loadSchedule(getDefaultSearchCriteria());
			setScheduleView(MONTH);
			viewType = "calendarView";
		} catch (Exception e) {
			log.error("Error in finding Events:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void eventSummaryAjax() {
		lastViewType = "eventView";
		resetMessages();
		eventTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("calendarListForm:eventList");
		currentMonth();
		viewType = "eventView";
	}

	public void exportCalendarDates() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.DAY_OF_YEAR, 1);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		Date firstOfCurrentYear = cal.getTime();
		cal.add(Calendar.YEAR, -7);
		exportMinYear = cal.getTime();
		cal.setTime(firstOfCurrentYear);
		cal.add(Calendar.YEAR, 8);
		cal.add(Calendar.DATE, -1);
		exportMaxYear = cal.getTime();
	}

	public void currentYearCalendars() {
		minCalDate = CommonUtility.dateToString(getMinCurrentYear(), "MM/dd/yyyy");
		maxCalDate = CommonUtility.dateToString(getMaxCurrentYear(), "MM/dd/yyyy");
	}

	public void nextYearCalendars() {
		minCalDate = CommonUtility.dateToString(getMinNextYear(), "MM/dd/yyyy");
		maxCalDate = CommonUtility.dateToString(getMaxNextYear(), "MM/dd/yyyy");
	}

	public void loadCalendarDates() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.DAY_OF_YEAR, 1);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		minCurrentYear = cal.getTime();
		cal.add(Calendar.YEAR, 1);
		minNextYear = cal.getTime();
		cal.add(Calendar.DATE, -1);
		maxCurrentYear = cal.getTime();
		cal.add(Calendar.YEAR, 1);
		maxNextYear = cal.getTime();
		minCalDate = CommonUtility.dateToString(getMinCurrentYear(), "MM/dd/yyyy");
		maxCalDate = CommonUtility.dateToString(getMaxCurrentYear(), "MM/dd/yyyy");
		setSelectDate(new Date());
	}

	// ******************************* AJAX Actions End
	// ***********************************************

	// Export Functionality View
	public void exportDisplay() {
		resetMessages();
		lastViewType = viewType;
		setExportStartDate(new Date());
		setExportEndDate(new Date());
		exportCalCateories = new ArrayList<CategoryDTO>();
		exportSelectedCategories = new String[50];
		for (CategoryDTO catg : calCategories) {
			exportCalCateories.add(catg);
		}
		int i = 0;
		for (String seletCatg : selectedCategories) {
			exportSelectedCategories[i] = seletCatg;
			i++;
		}
		lastViewType = viewType;
		viewType = "exportView";
	}
	
	public void admin() {
		viewType = "adminView";
	}
	

	public void validateExportCriteria() {
		exportValidationError = false;
		if (exportStartDate == null || exportEndDate == null || exportStartDate.after(exportEndDate)) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "The selected date range is invalid for producing an export.", ERR_PRCS_RQST));
			exportValidationError = true;
			return;
		}
		
		boolean catgFound=false;
		for(String catg: exportSelectedCategories){
			if(catg != null && catg.trim().length() > 0){
				catgFound=true;
				break;
			}
		}
		
		if(!catgFound){
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "There are no calendar events to export within the selected date range and categories.", ERR_PRCS_RQST));
			exportValidationError = true;
			return;
		}
		
	}

	public void exportCriteriaListener() {
		if (exportStartDate == null || exportEndDate == null || exportStartDate.after(exportEndDate)) {
			exportValidationError = true;
		} else {
			exportValidationError = false;
		}
	}


	public void updateView() {
		List<String> eventTypes = new ArrayList<String>();
		List<CategoryDTO> categoryDTOS = new ArrayList<CategoryDTO>();
		for (String category : selectedCategories) {
			if (category != null && category.trim().length() > 0) {
				eventTypes.add(category);
				CategoryDTO dto = new CategoryDTO();
				dto.setCategory(category);
				dto.setChecked(Boolean.TRUE);
				categoryDTOS.add(dto);
			} else {
				break;
			}
		}
		searchCriteria.setEventTypes(eventTypes);
		try {
			loadSchedule(searchCriteria);
		} catch (Exception e) {
			log.error("Error updating view:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	private List<String> getSelectedEventTypes() {
		List<String> eventTypes = new ArrayList<String>();
		for (String category : selectedCategories) {
			if (category != null && category.trim().length() > 0) {
				eventTypes.add(category);
			} else {
				break;
			}
		}
		return eventTypes;
	}


	public void backToCalendar(){
		viewType = "calendarView";
	}
	
	
	public void backToCalendarAdmin(){
		resetMessages();
		viewType = "adminView";
	}

	public void printHealth() {
		lastViewType = viewType;
		schedule = (Schedule) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("calendarSummaryForm:schedule");
		if(schedule.getView().equalsIgnoreCase(MONTH)){
			setScheduleDate(viewDisplayDate);
		}
		
		viewType = "printHealthView";
	}

	public void printEventDetail() {
		viewType = "printEventView";
	}

	public void printGenericEventDetail() {
		viewType = "printGenericEventView";
	}

	public void printEventSummary() {
		eventTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("calendarListForm:eventList");
		viewType = "printEventSummaryView";
	}

	public void printTaskDetail() {
		viewType = "printTaskView";
	}

	public void jumpToDate() {
		setSelectDate(new Date());
		minCalDate = CommonUtility.dateToString(getMinCurrentYear(), "MM/dd/yyyy");
		maxCalDate = CommonUtility.dateToString(getMaxCurrentYear(), "MM/dd/yyyy");
		lastViewType = viewType;
		viewType = "selectDateView";
	}

	public void changeDate() {
		setScheduleDate(selectDate);
		setScheduleView(AGENDA_DAY);
		lastViewType = viewType;
		viewType = "calendarView";
	}
	
	public void dateChange() {
		setScheduleDate(selectDate);
		setScheduleView(AGENDA_DAY);
		lastViewType = viewType;
		viewType = "calendarView";
	}
	
	public void deleteCategoryDiplay(){
		viewType = "deleteEventCategoryView";
	}
	

	public void currentMonth() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		Date startDate = cal.getTime();
		cal.add(Calendar.MONTH, 1);
		cal.add(Calendar.DAY_OF_MONTH, -1);

		Date endDate = cal.getTime();

		searchCriteria.setRangeStart(startDate);
		searchCriteria.setRangeEnd(endDate);
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setViewType("listView");
		searchCriteria.setKeyword(null);
		listselection="curr_month";
		try {
			loadSchedule(searchCriteria);
		} catch (MHVException e) {
			log.error("Error Loading Calendar:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void nextMonth() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		cal.add(Calendar.MONTH, 1);
		Date startDate = cal.getTime();
		cal.add(Calendar.MONTH, 1);
		cal.add(Calendar.DAY_OF_MONTH, -1);

		Date endDate = cal.getTime();
		searchCriteria.setRangeStart(startDate);
		searchCriteria.setRangeEnd(endDate);
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setViewType("listView");
		searchCriteria.setKeyword(null);
		listselection="next_month";
		try {
			loadSchedule(searchCriteria);
		} catch (MHVException e) {
			log.error("Error Loading Calendar:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void currentYear() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		Date startDate = cal.getTime();
		cal.add(Calendar.YEAR, 1);
		cal.add(Calendar.DAY_OF_MONTH, -1);

		Date endDate = cal.getTime();
		searchCriteria.setRangeStart(startDate);
		searchCriteria.setRangeEnd(endDate);
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setViewType("listView");
		searchCriteria.setKeyword(null);
		listselection="curr_year";
		try {
			loadSchedule(searchCriteria);
		} catch (MHVException e) {
			log.error("Error Loading Calendar:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void nextYear() {
		Calendar cal = Calendar.getInstance();
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		cal.add(Calendar.YEAR, 1);
		Date startDate = cal.getTime();
		cal.add(Calendar.YEAR, 1);
		cal.add(Calendar.DAY_OF_MONTH, -1);

		Date endDate = cal.getTime();
		searchCriteria.setRangeStart(startDate);
		searchCriteria.setRangeEnd(endDate);
		searchCriteria.setEventTypes(getSelectedEventTypes());
		searchCriteria.setViewType("listView");
		searchCriteria.setKeyword(null);
		listselection="next_year";
		try {
			loadSchedule(searchCriteria);
		} catch (MHVException e) {
			log.error("Error Loading Calendar:", e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST, ERR_PRCS_RQST));
		}
	}

	public void addEventStartDateChange() {
		newEvent.setFinishDate(newEvent.getBeginDate());
	}

	public void editEventStartDateChange() {
		selectedEvent.setFinishDate(selectedEvent.getBeginDate());
	}

	public ScheduleModel getLazyEventModel() {
		return lazyEventModel;
	}

	public void setLazyEventModel(ScheduleModel lazyEventModel) {
		this.lazyEventModel = lazyEventModel;
	}


	public MHVEventModel getEvent() {
		return event;
	}

	public void setEvent(MHVEventModel event) {
		this.event = event;
	}

	public MHVEventModel getSelectedEvent() {
		return selectedEvent;
	}

	public void setSelectedEvent(MHVEventModel selectedEvent) {
		this.selectedEvent = selectedEvent;
	}

	public String getSearchEventText() {
		return searchEventText;
	}

	public void setSearchEventText(String searchEventText) {
		this.searchEventText = searchEventText;
	}

	public Schedule getSchedule() {
		return schedule;
	}

	public void setSchedule(Schedule schedule) {
		this.schedule = schedule;
	}

	public String getScheduleView() {
		return scheduleView;
	}

	public void setScheduleView(String scheduleView) {
		this.scheduleView = scheduleView;
	}

	public Date getScheduleDate() {
		return scheduleDate;
	}

	public void setScheduleDate(Date scheduleDate) {
		this.scheduleDate = scheduleDate;
	}

	public MHVEventModel getNewEvent() {
		return newEvent;
	}

	public void setNewEvent(MHVEventModel newEvent) {
		this.newEvent = newEvent;
	}

	public String[] getSelectedCategories() {
		return selectedCategories;
	}

	public void setSelectedCategories(String[] selectedCategories) {
		this.selectedCategories = selectedCategories;
	}

	public ScheduleModel getEventModel() {
		return eventModel;
	}
	
	
	public Date getCalMonthDisplayDate(Date startDate){
		Calendar cal = Calendar.getInstance();
		cal.setTime(startDate);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		cal.add(Calendar.MONTH, -1);
		cal.add(Calendar.DATE, 14);
		return cal.getTime();
		
	}
	
	public Date getCalWeekDisplayDate(Date startDate){
		Calendar cal = Calendar.getInstance();
		cal.setTime(startDate);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		return cal.getTime();
		
	}
	
	public Date getCalDayDisplayDate(Date startDate){
		Calendar cal = Calendar.getInstance();
		cal.setTime(startDate);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		return cal.getTime();
		
	}
	
	private boolean isCurrentMonth(Date date){
		return CommonUtility.dateToString(date, "MMyyyy").equals(CommonUtility.dateToString(new Date(), "MMyyyy"));
	}
	
	private boolean isCurrentWeek(Date start,Date end){
		
		Calendar cal = Calendar.getInstance();
		cal.setTime(start);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.MILLISECOND,0);
		Date startDate=cal.getTime();
		
		cal.setTime(end);
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.MILLISECOND,0);
		
		Date endDate=cal.getTime();
		
		cal.setTime(new Date());
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.MILLISECOND,0);
		
		Date currDate= cal.getTime();
		
		return currDate.getTime() >= startDate.getTime() &&
				currDate.getTime() <= endDate.getTime();
	}


	public void setEventModel(ScheduleModel eventModel) {
		this.eventModel = eventModel;
	}

	public CalendarSearchCriteriaDTO getSearchCriteria() {
		return searchCriteria;
	}

	public void setSearchCriteria(CalendarSearchCriteriaDTO searchCriteria) {
		this.searchCriteria = searchCriteria;
	}

	public List<MHVEventModel> getEvents() {
		return events;
	}

	public void setEvents(List<MHVEventModel> events) {
		this.events = events;
	}

	public int getEventRowsPerPage() {
		return eventRowsPerPage;
	}

	public void setEventRowsPerPage(int eventRowsPerPage) {
		this.eventRowsPerPage = eventRowsPerPage;
	}

	public ValueExpression getEventSortColumn() {
		return eventSortColumn;
	}

	public void setEventSortColumn(ValueExpression eventSortColumn) {
		this.eventSortColumn = eventSortColumn;
	}

	public String getEventsortBy() {
		return eventsortBy;
	}

	public void setEventsortBy(String eventsortBy) {
		this.eventsortBy = eventsortBy;
	}

	public DataTable getEventTable() {
		return eventTable;
	}

	public void setEventTable(DataTable eventTable) {
		this.eventTable = eventTable;
	}

	public List<CategoryDTO> getCalCategories() {
		return calCategories;
	}

	public void setCalCategories(List<CategoryDTO> calCategories) {
		this.calCategories = calCategories;
	}

	public UserCalViewDTO getUserPreference() {
		return userPreference;
	}

	public void setUserPreference(UserCalViewDTO userPreference) {
		this.userPreference = userPreference;
	}

	public String getViewType() {
		return viewType;
	}

	public void setViewType(String viewType) {
		this.viewType = viewType;
	}

	public String getPreviousView() {
		return previousView;
	}

	public void setPreviousView(String previousView) {
		this.previousView = previousView;
	}

	public String getUserPreferView() {
		return userPreferView;
	}

	public void setUserPreferView(String userPreferView) {
		this.userPreferView = userPreferView;
	}

	public List<MHVEventModel> getSearchResults() {
		return searchResults;
	}

	public void setSearchResults(List<MHVEventModel> searchResults) {
		this.searchResults = searchResults;
	}

	public int getSearchResultsRowsPerPage() {
		return searchResultsRowsPerPage;
	}

	public void setSearchResultsRowsPerPage(int searchResultsRowsPerPage) {
		this.searchResultsRowsPerPage = searchResultsRowsPerPage;
	}

	public Date getSelectDate() {
		return selectDate;
	}

	public void setSelectDate(Date selectDate) {
		this.selectDate = selectDate;
	}

	public Date getMinCurrentYear() {
		return minCurrentYear;
	}

	public void setMinCurrentYear(Date minCurrentYear) {
		this.minCurrentYear = minCurrentYear;
	}

	public Date getMaxCurrentYear() {
		return maxCurrentYear;
	}

	public void setMaxCurrentYear(Date maxCurrentYear) {
		this.maxCurrentYear = maxCurrentYear;
	}

	public Date getMinNextYear() {
		return minNextYear;
	}

	public void setMinNextYear(Date minNextYear) {
		this.minNextYear = minNextYear;
	}

	public Date getMaxNextYear() {
		return maxNextYear;
	}

	public void setMaxNextYear(Date maxNextYear) {
		this.maxNextYear = maxNextYear;
	}

	public String getMinCalDate() {
		return minCalDate;
	}

	public void setMinCalDate(String minCalDate) {
		this.minCalDate = minCalDate;
	}

	public String getMaxCalDate() {
		return maxCalDate;
	}

	public void setMaxCalDate(String maxCalDate) {
		this.maxCalDate = maxCalDate;
	}

	public Integer getSearchResultNumber() {
		return searchResultNumber;
	}

	public void setSearchResultNumber(Integer searchResultNumber) {
		this.searchResultNumber = searchResultNumber;
	}

	public String getLastupdatedDate() {
		return lastupdatedDate;
	}

	public void setLastupdatedDate(String lastupdatedDate) {
		this.lastupdatedDate = lastupdatedDate;
	}

	public Date getExportStartDate() {
		return exportStartDate;
	}

	public void setExportStartDate(Date exportStartDate) {
		this.exportStartDate = exportStartDate;
	}

	public Date getExportEndDate() {
		return exportEndDate;
	}

	public void setExportEndDate(Date exportEndDate) {
		this.exportEndDate = exportEndDate;
	}

	public String[] getExportSelectedCategories() {
		return exportSelectedCategories;
	}

	public void setExportSelectedCategories(String[] exportSelectedCategories) {
		this.exportSelectedCategories = exportSelectedCategories;
	}

	public List<CategoryDTO> getExportCalCateories() {
		return exportCalCateories;
	}

	public void setExportCalCateories(List<CategoryDTO> exportCalCateories) {
		this.exportCalCateories = exportCalCateories;
	}

	public Date getExportMinYear() {
		return exportMinYear;
	}

	public void setExportMinYear(Date exportMinYear) {
		this.exportMinYear = exportMinYear;
	}

	public Date getExportMaxYear() {
		return exportMaxYear;
	}

	public void setExportMaxYear(Date exportMaxYear) {
		this.exportMaxYear = exportMaxYear;
	}

	public String getExportEventTypes() {
		boolean firstTime = true;
		StringBuffer sb = new StringBuffer();
		List<CategoryDTO> categorys =getCalCategories();
		for (String catg : exportSelectedCategories) {
			if (catg != null) {
				for(CategoryDTO category:categorys){
					if(catg.equals(category.getCategory())){
						if(!firstTime){
							sb.append(",");
							sb.append(category.getCategoryId());
						}else{
							sb.append(category.getCategoryId());
						}
						firstTime=false;
						break;	
					}
				}
			}
		}
		return sb.toString();
	}

	public String getStrExportStartDate() {
		return CommonUtility.dateToString(getExportStartDate(), "MMddyyyy");
	}

	public String getStrExportEndDate() {
		return CommonUtility.dateToString(getExportEndDate(), "MMddyyyy");
	}

	public boolean isExportValidationError() {
		return exportValidationError;
	}

	public void setExportValidationError(boolean exportValidationError) {
		this.exportValidationError = exportValidationError;
	}

	public String getServerTimeZone() {
		TimeZone tz = TimeZone.getDefault();
		return tz.getID();
	}

	public String getView() {
		return view;
	}

	public void setView(String view) {
		this.view = view;
	}

	public List<VisnDTO> getVisns() {
		return visns;
	}

	public void setVisns(List<VisnDTO> visns) {
		this.visns = visns;
	}

	public CalendarCategoryDTO getNewCategory() {
		return newCategory;
	}

	public void setNewCategory(CalendarCategoryDTO newCategory) {
		this.newCategory = newCategory;
	}

	public CalendarCategoryDTO getSelectedCategory() {
		return selectedCategory;
	}

	public void setSelectedCategory(CalendarCategoryDTO selectedCategory) {
		this.selectedCategory = selectedCategory;
	}

	public Date getViewDisplayDate() {
		return viewDisplayDate;
	}

	public void setViewDisplayDate(Date viewDisplayDate) {
		this.viewDisplayDate = viewDisplayDate;
	}


	public String getListselection() {
		return listselection;
	}


	public void setListselection(String listselection) {
		this.listselection = listselection;
	}

}