package gov.va.med.mhv.admin.web.formbean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.BeansException;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.model.User;
import com.liferay.portal.util.PortalUtil;

import gov.va.med.mhv.admin.dto.EmployeeDTO;
import gov.va.med.mhv.admin.dto.OrgDTO;
import gov.va.med.mhv.admin.service.mhvusers.FacilityInfoService;
import gov.va.med.mhv.admin.service.vastaff.EmployeeRoleManagementService;
import gov.va.med.mhv.admin.web.util.SessionUtil;
import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.exception.MHVException;

@ManagedBean(name = "searchUserIPA")
@ViewScoped
public class SearchUserIPAFormBean implements Serializable {

	private static final long serialVersionUID = -9012081572552097119L;

	private static Logger log = LogManager.getLogger(SearchUserIPAFormBean.class);

	private String firstName = "";
	private String lastName = "";
	private String middleName = "";
	private String userName;
	private String email;
	private String icn;
	private String ssn;
	private String ssn2;
	private String ssn3;
	private String ssn4;
	private String mhvId;
	private String firstPlusFour;
	private Date birthDate;
	private String[] matchingStatus;
	private String[] correlationStatus;
	private String[] authenticationStatus;
	private Date authenticationDateFrom;
	private Date authenticationDateTo;
	private String authenticatingFacility;
	private String treatingFacility;

	private String delayReason;
	private boolean ipaIdentificationPresented;
	private boolean ipaParticipationFormSigned;
	private boolean ipaApprovedForRecordsAccess;

	private boolean ipaStatusMatched;
	private boolean ipaStatusUnmatched;

	private boolean ipaStatusCorrelated;
	private boolean ipaStatusUncorrelated;
	private boolean ipaStatusPendingCorrelation;
	private boolean ipaStatusPendingUncorrelated;
	private boolean ipaStatusFailedCorrelation;
	private boolean ipaStatusFailedUncorrelation;
	private boolean ipaStatusCorrelationInvalid;
	private boolean ipaStatusAuthenticated;
	private boolean ipaStatusPrerequisitesComplete;
	private boolean ipaStatusInProcess;
	private boolean ipaStatusUnauthenticated;
	private boolean ipaStatusPendingDataMismatch;
	private boolean ipaStatusPendingUnauthentication;

	private String removeAuthenticatingReason;

	private Date latestDate;

	Map<String, String> facilities = null;
	List<OrgDTO> organizations = null;

	@ManagedProperty(value = "#{sessionUtil}")
	SessionUtil sessionUtil;

	@ManagedProperty(value = "#{facilityInfoProxy}")
	private FacilityInfoService facilityInfoProxy;

	@ManagedProperty(value = "#{employeeRoleManagementServiceProxy}")
	private EmployeeRoleManagementService employeeRoleManagementServiceProxy;

	// Generated by Map
	private static Map<String, Object> mStatusValues;
	static {
		mStatusValues = new LinkedHashMap<String, Object>();
		mStatusValues.put("Matched", "Matched"); // label, value
		mStatusValues.put("Unmatched", "Unmatched");
	}

	public Map<String, Object> getMatchingStatusValues() {
		return mStatusValues;
	}

	private static Map<String, Object> cStatusValues;
	static {
		cStatusValues = new LinkedHashMap<String, Object>();
		cStatusValues.put("Correlated", "2"); // label, value
		cStatusValues.put("Uncorrelated", "0");
		cStatusValues.put("Pending Correlation", "1");
		cStatusValues.put("Pending Uncorrelation", "3");
		cStatusValues.put("Failed Correlation", "4");
		cStatusValues.put("Failed Uncorrelation", "5");
		cStatusValues.put("Correlation Invalid", "6");
	}

	public Map<String, Object> getCorrelationStatusValues() {
		return cStatusValues;
	}

	private static Map<String, Object> aStatusValues;
	static {
		aStatusValues = new LinkedHashMap<String, Object>();
		aStatusValues.put("Authenticated", "Authenticated"); // label, value
		aStatusValues.put("Unauthenticated", "Unauthenticated");
		aStatusValues.put("Prerequisites Complete", "PrerequisitesComplete");
		aStatusValues.put("Pending Data Mismatch", "PendingDataMismatch");
		aStatusValues.put("In Process", "InProcess");
		aStatusValues.put("Unauthentication Unsuccessful", "UnauthenticationUnsuccessful");
	}

	public Map<String, Object> getAuthenticationStatusValues() {
		return aStatusValues;
	}

	private static Map<String, Object> adelayReasons;
	static {
		adelayReasons = new LinkedHashMap<String, Object>();
		adelayReasons.put("Prerequisites Incomplete", "PreRequisitesIncomplete"); // label,
																					// value
		adelayReasons.put("Sensitive Records Check Incomplete", "SensitiveRecordsCheckIncomplete");
	}

	public Map<String, Object> getDelayReasons() {
		return adelayReasons;
	}

	private static Map<String, Object> aremoveReasons;
	static {
		aremoveReasons = new LinkedHashMap<String, Object>();
		aremoveReasons.put("Mistaken ID", "MistakenId"); // label, value
		aremoveReasons.put("Patient Safety", "PatientSafety");
		aremoveReasons.put("Administrative Error", "AdministrativeError");
	}

	public Map<String, Object> getRemoveReasons() {
		return aremoveReasons;
	}

	public void setFacilities(Map<String, String> facilities) {
		this.facilities = facilities;
	}

	// Get a list of Facilities for an employee
	public Map<String, String> getFacilities() {
		try {
			if (facilities == null || facilities.isEmpty()) {
				List<OrgDTO> allOrgs = getOrganizations();
				facilities = new LinkedHashMap<String, String>();
				for (OrgDTO org : allOrgs) {
					FacilityInfoDTO fInfo = facilityInfoProxy.getFacilityInfoById(org.getFacilityInfoId());
					facilities.put(fInfo.getName(), Long.toString(fInfo.getId()));
				}
			}
		} catch (MHVException e) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
			log.error("Error in getFacilities for Admin user name: ", getAdminUserName(), e);
		}
		return facilities;
	}

	// Get a list of Organizations for an employee
	private List<OrgDTO> getOrganizations() {
		if (organizations == null || organizations.isEmpty()) {
			Long employeeId = null;
			try {

				String loggedInuserName = getAdminUserName();
				EmployeeDTO currentUser = sessionUtil.getEmployee(loggedInuserName);

				if (currentUser != null) {
					employeeId = currentUser.getEmployeeId();
					organizations = employeeRoleManagementServiceProxy
							.getOrganizationsOfTypeWhereEmployeeHasRole(employeeId, "Facility", "Authenticator");
				}
			} catch (BeansException e) {
				FacesContext.getCurrentInstance().addMessage("Application Error.",
						new FacesMessage("Failure loading Facility list."));
				log.error("Error loading Facility list for employeeId: " + employeeId, e);
			} catch (MHVException e) {
				FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,
						"Unable to load Facility List: for employeeId: " + employeeId + e.getFailureMessage(), null));
				log.error("Error loading Facility list for employeeId: " + employeeId, e);
			}
		}
		if (organizations == null) {
			organizations = new ArrayList<OrgDTO>(0);
		}
		return organizations;
	}

	public String getAdminUserName() {
		String adminUserNameStr = "";
		User liferayUser = null;
		PortletRequest request = (PortletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		HttpServletRequest httpRequest = PortalUtil.getHttpServletRequest(request);
		HttpSession session = httpRequest.getSession();
		adminUserNameStr = (String) session.getAttribute("LIFERAY_SHARED_screenName");
		if (StringUtils.isBlank(adminUserNameStr)) {
			try {
				liferayUser = PortalUtil.getUser(request);
				if (liferayUser != null) {
					adminUserNameStr = liferayUser.getScreenName();
				}
			} catch (PortalException e) {
				e.printStackTrace();
			} catch (SystemException e) {
				e.printStackTrace();
			}
		}
		return adminUserNameStr;
		// return "vacojonnag";
	}

	public boolean getIpaIdentificationPresented() {
		return ipaIdentificationPresented;
	}

	public void setIpaIdentificationPresented(boolean ipaIdentificationPresented) {
		this.ipaIdentificationPresented = ipaIdentificationPresented;
	}

	public boolean getIpaParticipationFormSigned() {
		return ipaParticipationFormSigned;
	}

	public void setIpaParticipationFormSigned(boolean ipaParticipationFormSigned) {
		this.ipaParticipationFormSigned = ipaParticipationFormSigned;
	}

	public boolean getIpaApprovedForRecordsAccess() {
		return ipaApprovedForRecordsAccess;
	}

	public void setIpaApprovedForRecordsAccess(boolean ipaApprovedForRecordsAccess) {
		this.ipaApprovedForRecordsAccess = ipaApprovedForRecordsAccess;
	}

	public String getRemoveAuthenticatingReason() {
		return removeAuthenticatingReason;
	}

	public void setRemoveAuthenticatingReason(String removeAuthenticatingReason) {
		this.removeAuthenticatingReason = removeAuthenticatingReason;
	}

	public String getDelayReason() {
		return delayReason;
	}

	public void setDelayReason(String delayReason) {
		this.delayReason = delayReason;
	}

	public FacilityInfoService getFacilityInfoProxy() {
		return facilityInfoProxy;
	}

	public void setFacilityInfoProxy(FacilityInfoService facilityInfoProxy) {
		this.facilityInfoProxy = facilityInfoProxy;
	}

	public EmployeeRoleManagementService getEmployeeRoleManagementServiceProxy() {
		return employeeRoleManagementServiceProxy;
	}

	public void setEmployeeRoleManagementServiceProxy(
			EmployeeRoleManagementService employeeRoleManagementServiceProxy) {
		this.employeeRoleManagementServiceProxy = employeeRoleManagementServiceProxy;
	}

	public boolean isIpaStatusMatched() {
		return ipaStatusMatched;
	}

	public void setIpaStatusMatched(boolean ipaStatusMatched) {
		this.ipaStatusMatched = ipaStatusMatched;
	}

	public boolean isIpaStatusUnmatched() {
		return ipaStatusUnmatched;
	}

	public void setIpaStatusUnmatched(boolean ipaStatusUnmatched) {
		this.ipaStatusUnmatched = ipaStatusUnmatched;
	}

	public boolean isIpaStatusCorrelated() {
		return ipaStatusCorrelated;
	}

	public void setIpaStatusCorrelated(boolean ipaStatusCorrelated) {
		this.ipaStatusCorrelated = ipaStatusCorrelated;
	}

	public boolean isIpaStatusUncorrelated() {
		return ipaStatusUncorrelated;
	}

	public void setIpaStatusUncorrelated(boolean ipaStatusUncorrelated) {
		this.ipaStatusUncorrelated = ipaStatusUncorrelated;
	}

	public boolean isIpaStatusPendingCorrelation() {
		return ipaStatusPendingCorrelation;
	}

	public void setIpaStatusPendingCorrelation(boolean ipaStatusPendingCorrelation) {
		this.ipaStatusPendingCorrelation = ipaStatusPendingCorrelation;
	}

	public boolean isIpaStatusPendingUncorrelated() {
		return ipaStatusPendingUncorrelated;
	}

	public void setIpaStatusPendingUncorrelated(boolean ipaStatusPendingUncorrelated) {
		this.ipaStatusPendingUncorrelated = ipaStatusPendingUncorrelated;
	}

	public boolean isIpaStatusFailedCorrelation() {
		return ipaStatusFailedCorrelation;
	}

	public void setIpaStatusFailedCorrelation(boolean ipaStatusFailedCorrelation) {
		this.ipaStatusFailedCorrelation = ipaStatusFailedCorrelation;
	}

	public boolean isIpaStatusFailedUncorrelation() {
		return ipaStatusFailedUncorrelation;
	}

	public void setIpaStatusFailedUncorrelation(boolean ipaStatusFailedUncorrelation) {
		this.ipaStatusFailedUncorrelation = ipaStatusFailedUncorrelation;
	}

	public boolean isIpaStatusCorrelationInvalid() {
		return ipaStatusCorrelationInvalid;
	}

	public void setIpaStatusCorrelationInvalid(boolean ipaStatusCorrelationInvalid) {
		this.ipaStatusCorrelationInvalid = ipaStatusCorrelationInvalid;
	}

	public boolean isIpaStatusAuthenticated() {
		return ipaStatusAuthenticated;
	}

	public void setIpaStatusAuthenticated(boolean ipaStatusAuthenticated) {
		this.ipaStatusAuthenticated = ipaStatusAuthenticated;
	}

	public boolean isIpaStatusPrerequisitesComplete() {
		return ipaStatusPrerequisitesComplete;
	}

	public void setIpaStatusPrerequisitesComplete(boolean ipaStatusPrerequisitesComplete) {
		this.ipaStatusPrerequisitesComplete = ipaStatusPrerequisitesComplete;
	}

	public boolean isIpaStatusInProcess() {
		return ipaStatusInProcess;
	}

	public void setIpaStatusInProcess(boolean ipaStatusInProcess) {
		this.ipaStatusInProcess = ipaStatusInProcess;
	}

	public boolean isIpaStatusUnauthenticated() {
		return ipaStatusUnauthenticated;
	}

	public void setIpaStatusUnauthenticated(boolean ipaStatusUnauthenticated) {
		this.ipaStatusUnauthenticated = ipaStatusUnauthenticated;
	}

	public boolean isIpaStatusPendingDataMismatch() {
		return ipaStatusPendingDataMismatch;
	}

	public void setIpaStatusPendingDataMismatch(boolean ipaStatusPendingDataMismatch) {
		this.ipaStatusPendingDataMismatch = ipaStatusPendingDataMismatch;
	}

	public boolean isIpaStatusPendingUnauthentication() {
		return ipaStatusPendingUnauthentication;
	}

	public void setIpaStatusPendingUnauthentication(boolean ipaStatusPendingUnauthentication) {
		this.ipaStatusPendingUnauthentication = ipaStatusPendingUnauthentication;
	}

	public Date getLatestDate() {
		latestDate = new Date();
		return latestDate;
	}

	public void setLatestDate(Date latestDate) {
		this.latestDate = latestDate;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getIcn() {
		return icn;
	}

	public void setIcn(String icn) {
		this.icn = icn;
	}

	public String getSsn() {
		return ssn;
	}

	public void setSsn(String ssn) {
		this.ssn = ssn;
	}

	public String getSsn2() {
		return ssn2;
	}

	public void setSsn2(String ssn2) {
		this.ssn2 = ssn2;
	}

	public String getSsn3() {
		return ssn3;
	}

	public void setSsn3(String ssn3) {
		this.ssn3 = ssn3;
	}

	public String getSsn4() {
		return ssn4;
	}

	public void setSsn4(String ssn4) {
		this.ssn4 = ssn4;
	}

	public String getFirstPlusFour() {
		return firstPlusFour;
	}

	public void setFirstPlusFour(String firstPlusFour) {
		this.firstPlusFour = firstPlusFour;
	}

	public Date getBirthDate() {
		return birthDate;
	}

	public void setBirthDate(Date birthDate) {
		this.birthDate = birthDate;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getMhvId() {
		return mhvId;
	}

	public void setMhvId(String mhvId) {
		this.mhvId = mhvId;
	}

	public String[] getMatchingStatus() {
		return matchingStatus;
	}

	public void setMatchingStatus(String[] matchingStatus) {
		this.matchingStatus = matchingStatus;
	}

	public String[] getCorrelationStatus() {
		return correlationStatus;
	}

	public void setCorrelationStatus(String[] correlationStatus) {
		this.correlationStatus = correlationStatus;
	}

	public String[] getAuthenticationStatus() {
		return authenticationStatus;
	}

	public void setAuthenticationStatus(String[] authenticationStatus) {
		this.authenticationStatus = authenticationStatus;
	}

	public Date getAuthenticationDateFrom() {
		return authenticationDateFrom;
	}

	public void setAuthenticationDateFrom(Date authenticationDateFrom) {
		this.authenticationDateFrom = authenticationDateFrom;
	}

	public Date getAuthenticationDateTo() {
		return authenticationDateTo;
	}

	public void setAuthenticationDateTo(Date authenticationDateTo) {
		this.authenticationDateTo = authenticationDateTo;
	}

	public String getAuthenticatingFacility() {
		return authenticatingFacility;
	}

	public void setAuthenticatingFacility(String authenticatingFacility) {
		this.authenticatingFacility = authenticatingFacility;
	}

	public String getTreatingFacility() {
		return treatingFacility;
	}

	public void setTreatingFacility(String treatingFacility) {
		this.treatingFacility = treatingFacility;
	}

	public SessionUtil getSessionUtil() {
		return sessionUtil;
	}

	public void setSessionUtil(SessionUtil sessionUtil) {
		this.sessionUtil = sessionUtil;
	}

}
